// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2009-2013 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Mono.Unix;

namespace Smdn.IO.FileSystems.Unix {
  public class UnixDrive : Drive {
    /*
     * class members
     */
    public static IEnumerable<UnixDrive> GetDrives()
    {
      foreach (var driveInfo in UnixDriveInfo.GetDrives()) {
        yield return new UnixDrive(driveInfo);
      }
    }

    /*
     * instance members
     */
    public override string Name {
      get { return info.VolumeLabel; }
    }

    public override System.IO.DriveType Type {
      get
      {
        switch (info.DriveType) {
          case UnixDriveType.Fixed:           return System.IO.DriveType.Fixed;
          case UnixDriveType.CDRom:           return System.IO.DriveType.CDRom;
          case UnixDriveType.Network:         return System.IO.DriveType.Network;
          case UnixDriveType.Removable:       return System.IO.DriveType.Removable;
          case UnixDriveType.Ram:             return System.IO.DriveType.Ram;
          case UnixDriveType.NoRootDirectory: return System.IO.DriveType.NoRootDirectory;
          default: return System.IO.DriveType.Unknown;
        }
      }
    }

    public override bool IsReady {
      get { return info.IsReady; }
    }

    public override Directory RootDirectory {
      get { return new UnixDirectory(info.RootDirectory); }
    }

    public override string Format {
      get { return info.DriveFormat; }
    }

    public override long TotalSize {
      get { return info.TotalSize; }
    }

    public override long TotalFreeSpace {
      get { return info.TotalFreeSpace; }
    }

    public override object RawInfo {
      get { return info; }
    }

    internal UnixDrive(UnixDriveInfo driveInfo)
    {
      if (driveInfo == null)
        throw new ArgumentNullException("driveInfo");

      this.info = driveInfo;
    }

    private UnixDriveInfo info;
  }
}