// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2009-2013 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;

namespace Smdn.IO.FileSystems.Windows {
  public class WindowsFile : File {
    public override string Name {
      get { return info.Name; }
    }

    public override long Length {
      get { return info.Length; }
    }

    public override bool Exists {
      get { return info.Exists; }
    }

    public override Directory Parent {
      get { return new WindowsDirectory(info.Directory); }
    }

    public override Directory Root {
      get { return new WindowsDirectory(info.Directory.Root); }
    }

    public override string FullName {
      get { return info.FullName; }
    }

    public override object RawInfo {
      get { return info; }
    }

    internal WindowsFile(FileInfo fileInfo)
    {
      if (fileInfo == null)
        throw new ArgumentNullException("fileInfo");

      this.info = fileInfo;
    }

    public override void Refresh()
    {
      info.Refresh();
    }

    private FileInfo info;
  }
}
