//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2003-2013 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Text;
using System.Xml;

using Smdn.Formats;
using Smdn.IO.FileSystems;

namespace Smdn.Applications.BittenBaum.Formatting {
  using Analysis;
  using Graphing;

  using IO = System.IO;

  partial class Formatter {
    /// <summary>
    /// XML用のFormatter
    /// </summary>
    private class XmlFormatter {
      private readonly FormattingContext.FormattingParameter parameter;

      public XmlFormatter(FormattingContext.FormattingParameter parameter)
      {
        this.parameter = parameter;
      }

      public void Format(FormattingContext.FormattingStatus status)
      {
        var writer = new IO.StreamWriter(parameter.OutputStream, Encoding.UTF8);

        var tree = parameter.Graph.DirectoryTree;

        status.TotalEntryCount = parameter.FormattingElements.Count;
        status.FormattedEntryCount = 0;

        var document = new XmlDocument();

        document.AppendChild(document.CreateXmlDeclaration("1.0", null, null));

        var root = document.AppendChild(document.CreateElement("analysis")) as XmlElement;

        root.SetAttribute("root", tree.FullName);

        var stat = root.AppendChild(document.CreateElement("statistics")) as XmlElement;

        stat.AppendChild(document.CreateElement("name")).AppendChild(document.CreateTextNode(tree.Name));
        stat.AppendChild(document.CreateElement("entries")).AppendChild(document.CreateTextNode(XmlConvert.ToString(tree.TotalEntryCount)));
        stat.AppendChild(document.CreateElement("files")).AppendChild(document.CreateTextNode(XmlConvert.ToString(tree.TotalFileCount)));
        stat.AppendChild(document.CreateElement("directories")).AppendChild(document.CreateTextNode(XmlConvert.ToString(tree.TotalSubTreeCount)));

        var totalSize = tree.GetTotalFileSize();

        stat.AppendChild(document.CreateElement("size")).AppendChild(document.CreateTextNode(XmlConvert.ToString(totalSize)));
        stat.AppendChild(document.CreateElement("sizeString")).AppendChild(document.CreateTextNode(string.Format(SIPrefixFormat.InvaliantInfo, "{0:b2}B", totalSize)));

        var result = root.AppendChild(document.CreateElement("result")) as XmlElement;

        result.SetAttribute("entries", parameter.FormattingElements.Count.ToString());

        foreach (var element in parameter.FormattingElements) {
          XmlElement ent;
          long size;
          var entry = element.Entry;

          if (entry is DirectoryTree) {
            ent = document.CreateElement("directory");
            size = (entry as DirectoryTree).GetTotalFileSize();
          }
          else if (entry is File) {
            ent = document.CreateElement("file");
            size = entry.Length;
          }
          else {
            continue;
          }

          result.AppendChild(ent);

          ent.AppendChild(document.CreateTextNode(entry.FullName));
          ent.SetAttribute("name", entry.Name);
          ent.SetAttribute("size", XmlConvert.ToString(size));
          ent.SetAttribute("sizePercent", element.Range.ToString("P2"));
          ent.SetAttribute("sizeString", string.Format(SIPrefixFormat.InvaliantInfo, "{0:b1}B", size));
          ent.SetAttribute("directory", entry.Parent.FullName);

          status.FormattedEntryCount++;
        }

        status.FormattedEntryCount = status.TotalEntryCount;

        var settings = new XmlWriterSettings();

        settings.Encoding = Encoding.UTF8;
        settings.Indent = true;
        settings.IndentChars = "  ";
        settings.CloseOutput = parameter.CloseStreamAfterFormatting;

        var xw = XmlWriter.Create(writer, settings);

        document.WriteTo(xw);

        xw.Flush();
        xw.Close();
      }
    }
  }
}