//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2003-2013 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Collections.Generic;
using System.IO;

namespace Smdn.Applications.BittenBaum.Formatting {
  using Analysis;
  using Graphing;

  public class FormattingContext : TaskContext, ITaskContext {
    public FormattingContext()
    {
      this.parameter = new FormattingParameter();
      this.status = new FormattingStatus();
      this.result = new FormattingResult();
    }

    #region parameters
    private readonly FormattingParameter parameter;

    public FormattingParameter Parameter {
      get { return parameter; }
    }

    public class FormattingParameter : TaskParameter
    {
      public FormatType Type = FormatType.Xml;
      public Graph Graph = null;
      // TODO: IReadOnlyCollection
      public ICollection<IGraphElement> FormattingElements = null;
      public Stream OutputStream = null;
      public bool CloseStreamAfterFormatting = true;
      public Comparison<IGraphElement> Comparison = null;
    }
    #endregion

    #region status
    private readonly FormattingStatus status;

    public FormattingStatus Status {
      get { return status; }
    }

    public class FormattingStatus : TaskStatus {
      public int TotalEntryCount = 0;
      public int FormattedEntryCount = 0;

      public override int Total {
        get { return TotalEntryCount; }
      }

      public override int Processed {
        get { return FormattedEntryCount; }
      }
    }
    #endregion

    #region results
    private readonly FormattingResult result;

    public FormattingResult Result {
      get { return result; }
    }

    public class FormattingResult : TaskResult {
    }
    #endregion

    #region ITaskContext

    TaskContext.TaskParameter ITaskContext.Parameter {
      get { return parameter; }
    }

    TaskContext.TaskResult ITaskContext.Result {
      get { return result; }
    }

    TaskContext.TaskStatus ITaskContext.Status {
      get { return status; }
    }

    #endregion
  }
}
