//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2003-2013 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;

using Smdn.IO.FileSystems;

namespace Smdn.Applications.BittenBaum.Graphing {
  using Analysis;

  public class GraphingContext : TaskContext, ITaskContext {
    public GraphingContext()
    {
      this.parameter = new GraphingParameter();
      this.status = new GraphingStatus();
      this.result = new GraphingResult();
    }

    #region parameters
    private readonly GraphingParameter parameter;

    public GraphingParameter Parameter {
      get { return parameter; }
    }

    public class GraphingParameter : TaskParameter
    {
      public GraphType Type = GraphType.Disc;
      public int MaxDepth = 0;
      public bool CalculateRangeByDriveUsage = false;
      public DirectoryTree DirectoryTree = null;
      public Comparison<DirectoryEntry> Comparison = DirectoryEntryComparison.GetComparison(DirectoryEntryCompareMode.BySize);
    }
    #endregion

    #region status
    private readonly GraphingStatus status;

    public GraphingStatus Status {
      get { return status; }
    }

    public class GraphingStatus : TaskStatus {
      public int TotalEntryCount = 0;
      public int GraphedEntryCount = 0;

      public override int Total {
        get { return TotalEntryCount; }
      }

      public override int Processed {
        get { return GraphedEntryCount; }
      }
    }
    #endregion

    #region results
    private readonly GraphingResult result;

    public GraphingResult Result {
      get { return result; }
    }

    public class GraphingResult : TaskResult {
      public Graph Graph = null;
    }
    #endregion

    #region ITaskContext

    TaskContext.TaskParameter ITaskContext.Parameter {
      get { return parameter; }
    }

    TaskContext.TaskResult ITaskContext.Result {
      get { return result; }
    }

    TaskContext.TaskStatus ITaskContext.Status {
      get { return status; }
    }

    #endregion
  }
}
