//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2003-2013 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

namespace Smdn.Applications.BittenBaum.Rendering {
  using Graphing;

  public partial class Renderer {
    private readonly RenderingContext context = null;

    public Renderer(RenderingContext context)
    {
      if (context == null)
        throw new ArgumentNullException("context");

      this.context = context;
    }

    public void Render()
    {
      var param = context.Parameter;
      var bakingBaumkuchen = param.ColoringMode == ColoringMode.DirectoryBaumkuchen;

      context.Status.StatusText = bakingBaumkuchen ? "バウムクーヘンを焼いています" : "グラフを描画中";

      context.Result.GraphImage = new Bitmap(param.OutputWidth, param.OutputHeight, PixelFormat.Format32bppArgb);

      using (var g = Graphics.FromImage(context.Result.GraphImage)) {
        // 透明でクリア
        g.Clear(Color.Transparent);

        g.SmoothingMode = param.SmoothingMode;
        g.InterpolationMode = param.InterpolationMode;
        g.TextRenderingHint = param.TextRenderingHint;

        // 描画領域
        switch (param.Graph.Type) {
          case GraphType.Disc:
            using (var renderer = new DiscRenderer(param)) {
              renderer.Render(g, context.Status);
            }
            break;

          default:
            // TODO: 円グラフ以外の形式はまだ
            break;
        }
      }

      context.Status.StatusText = bakingBaumkuchen ? "バウムクーヘンが焼けました" : "グラフの描画が完了しました";
    }

    public IGraphElement GetElementAt(PointF point)
    {
      switch (context.Parameter.Graph.Type) {
        case GraphType.Disc:
          using (var renderer = new DiscRenderer(context.Parameter)) {
            return renderer.GetPieAt(point);
          }

        default:
          // TODO: 円グラフ以外の形式はまだ
          return null;
      }
    }

    public PartialGraphImage GetElementImage(IEnumerable<IGraphElement> elements, bool highlighted)
    {
      switch (context.Parameter.Graph.Type) {
        case GraphType.Disc:
          using (var renderer = new DiscRenderer(context.Parameter)) {
            return renderer.GetPieImage(elements, highlighted);
          }

        default:
          // TODO: 円グラフ以外の形式はまだ
          return null;
      }
    }
  }
}
